import json
from moviepy.editor import VideoFileClip, TextClip, CompositeVideoClip
from moviepy.video.tools.subtitles import SubtitlesClip
from PIL import ImageFont

# Function to assign colors to speakers
def get_speaker_color(speaker, color_map):
    if speaker not in color_map:
        color_map[speaker] = f"hsl({len(color_map) * 60 % 360}, 100%, 50%)"
    return color_map[speaker]

# Function to parse the JSON and create subtitles
def parse_subtitles(json_file):
    with open(json_file, 'r') as f:
        data = json.load(f)

    subtitles = []
    color_map = {}

    for entry in data:
        start_time = entry["start_time"]
        end_time = entry["end_time"]
        speaker = entry["speaker"]
        text = entry["text"]
        
        color = get_speaker_color(speaker, color_map)
        subtitles.append(((start_time, end_time), f"{speaker}: {text}", color))
    
    return subtitles

# Function to generate text clips
def subtitle_generator(txt, color):
    return TextClip(txt, fontsize=24, font='Arial', color=color, bg_color='black')

# Main function to burn subtitles into the video
def burn_subtitles(input_video, subtitle_json, output_video):
    video = VideoFileClip(input_video)

    subtitles_data = parse_subtitles(subtitle_json)

    # Create subtitle clips
    subtitle_clips = []
    for ((start, end), txt, color) in subtitles_data:
        subtitle_clip = (TextClip(txt, fontsize=24, color=color, font='Arial', bg_color='black')
                         .set_position(('center', 'bottom'))
                         .set_start(start)
                         .set_end(end))
        subtitle_clips.append(subtitle_clip)
    
    # Overlay subtitles on the video
    final = CompositeVideoClip([video] + subtitle_clips)
    final.write_videofile(output_video, codec="libx264")

# Example usage
burn_subtitles('input.mp4', 'subtitles.json', 'output.mp4')

